	/* G N O K K I
	   A Linux/Unix toolset and driver for the Nokia 3110/3810/8110 mobiles.
	   Copyright (C) Hugh Blemings, 1999  Released under the terms of 
       the GNU GPL, see file COPYING for more details.
	
	   This file:  fbus.h   Version 0.2.1

	   Header file for the various functions, definitions etc. used
	   to implement the handset interface.  See fbus.c for more details. */



	/* Change FBUS_DEVICE to suit your serial port.  You must have
	   read/write permissions for this port. */
#define 	FBUS_DEVICE 				"/dev/ttyS0"

	/* Defines, enums etc. specific to fbus code. */
#define		FBUS_MAX_RECEIVE_LENGTH 			(512)
#define		FBUS_MAX_TRANSMIT_LENGTH			(256)
#define 	FBUS_BAUDRATE 						(B115200)

	/* Limits for sizing of array in FBUS_PhonebookEntry */
#define		FBUS_MAX_PHONEBOOK_NAME_LENGTH				(30)
#define		FBUS_MAX_PHONEBOOK_NUMBER_LENGTH			(30)

	/* These two set limits for the handset.  This is SIM dependant
	   and ultimately will be set automatically once we know how
	   to get that information from phone. */
#define		FBUS_DEFAULT_MAX_PHONEBOOK_NAME_LENGTH		(10)
#define		FBUS_DEFAULT_MAX_PHONEBOOK_NUMBER_LENGTH	(30)

	/* Limits to do with SMS messages. */
#define		FBUS_MAX_MSG_CENTRE_LENGTH					(30)
#define		FBUS_MAX_SENDER_LENGTH						(30)
#define		FBUS_MAX_SMS_LENGTH							(160)

	/* Define datatype for phonebook entry, used for getting/writing
	   phonebook entries. */
typedef struct {
	bool	Empty;
	char	Name[FBUS_MAX_PHONEBOOK_NAME_LENGTH + 1]; /* Plus 1 for null terminator. */
	char	Number[FBUS_MAX_PHONEBOOK_NUMBER_LENGTH + 1];
} FBUS_PhonebookEntry;

	/* Define datatype for SMS messages, used for getting SMS
	   messages from the phones memory and sending them to the 
	   GSM network. */
typedef struct {
	int		Year, Month, Day;	/* Time/date of reception of messages. */
	int		Hour, Minute, Second;
	int		Length;
	char	MessageText[FBUS_MAX_SMS_LENGTH + 1]; 	/* Room for null term. */
	char	MessageCentre[FBUS_MAX_MSG_CENTRE_LENGTH + 1];
	char	Sender[FBUS_MAX_SENDER_LENGTH + 1];
	int		MessageNumber;	/* Location in phones memory. */
	int		MemoryType;		/* Which memory type message is stored in. */

	u8		Unk2, Unk3, Unk4, Unk5, Unk9, UnkEnd;	/* Unknown bytes. */
} FBUS_SMSMessage;


	/* States for receive code. */
enum	FBUS_RX_States {FBUS_RX_Initialise,
						FBUS_RX_Sync,
						FBUS_RX_GetLength,
						FBUS_RX_GetMessage};

	/* Define standard FBUS error/return code values.  These codes
	   are also used for some internal functions such as SIM read/write */
typedef enum {FBE_NONE = 0,
			  FBE_NOLINK,
			  FBE_TIMEOUT,
			  FBE_INVALIDSMSLOCATION,
			  FBE_INVALIDPHBOOKLOCATION,
			  FBE_EMPTYSMSLOCATION,
			  FBE_PHBOOKNAMETOOLONG,
			  FBE_PHBOOKNUMBERTOOLONG,
			  FBE_PHBOOKWRITEFAILED,
			  FBE_INTERNALERROR,
			  FBE_BUSY,
		} FBUS_Error;

	/* Global variables */
extern bool					FBUS_LinkOK;
extern bool					FBUS_EnableMonitoringOutput;

	/* Prototypes */

	/* Prototypes for the functions designed to be used externally. */
bool		FBUS_Initialise(void);
bool		FBUS_OpenSerial(void);
void		FBUS_Terminate(void);

FBUS_Error	FBUS_GetPhonebookLocation(int memory_area, int location, FBUS_PhonebookEntry *entry);

FBUS_Error	FBUS_WritePhonebookLocation(int memory_area, int location, FBUS_PhonebookEntry *entry);

FBUS_Error	FBUS_GetSMSMessage(int memory_area, int location, FBUS_SMSMessage *message);

	/* Most of the following are for internal functions. */
void	FBUS_SigHandler(int status);
void	FBUS_ThreadLoop(void);

void	FBUS_RX_StateMachine(char rx_byte);
void	FBUS_RX_DispatchMessage(void);
void	FBUS_RX_DisplayMessage(void);

void    FBUS_RX_Handle0x0b_IncomingCall(void);
void    FBUS_RX_Handle0x4b_Status(void);
void    FBUS_RX_Handle0x10_EndOfOutgoingCall(void);
void    FBUS_RX_Handle0x11_EndOfIncomingCall(void);
void    FBUS_RX_Handle0x12_EndOfOutgoingCall(void);
void	FBUS_RX_Handle0x27_SMSMessageText(void);
void	FBUS_RX_Handle0x30_IncomingSMSNotification(void);
void    FBUS_RX_Handle0x0d_IncomingCallAnswered(void);
void    FBUS_RX_Handle0x0e_OutgoingCallAnswered(void);
void	FBUS_RX_Handle0x2c_SMSHeader(void);
void	FBUS_RX_Handle0x41_SMSMessageCentreData(void);
void    FBUS_RX_Handle0x46_MemoryLocationData(void);


void    FBUS_TX_UpdateSequenceNumber(void);
int     FBUS_TX_SendStandardAcknowledge(u8 message_type);

int		FBUS_TX_SendMessage(u8 message_length, u8 message_type, u8 sequence_byte, u8 *buffer);
void 	FBUS_TX_Send0x25_RequestSMSMemoryLocation(u8 memory_type, u8 location); 
void 	FBUS_TX_Send0x3fMessage(void); 
void 	FBUS_TX_Send0x4aMessage(void);
int		FBUS_TX_Send0x42_WriteMemoryLocation(u8 memory_area, u8 location, char *label, char *number);
void    FBUS_TX_Send0x43_RequestMemoryLocation(u8 memory_area, u8 location);
void    FBUS_TX_Send0x15Message(u8 sequence_number);



